/*----------------------------------------------------------------------*/
/*  Complex  - is a small library of complex arithmatic                 */
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <math.h>
#include "complex.h"

/*----------------------------------------------------------------------*/
/* .. C -- assigns a complex number                                     */
/*----------------------------------------------------------------------*/

C(c,r,i) 
complex *c;
double r,i;
{      
   c->r = r;
   c->i = i;
}

/*----------------------------------------------------------------------*/
/* .. AC -- adds complex numbers                                        */
/*----------------------------------------------------------------------*/

AC (result,c1, c2)
complex *result,*c1,*c2;
{
   result->r = c1->r + c2->r;
   result->i = c1->i + c2->i;
}               

/*----------------------------------------------------------------------*/
/* .. SC -- subtracts complex numbers                                   */
/*----------------------------------------------------------------------*/

SC (result,c1, c2)
complex *result,*c1,*c2;
{
   result->r = c1->r - c2->r;
   result->i = c1->i - c2->i;
}               

/*----------------------------------------------------------------------*/
/*  .. SmC -- Scalar multiplication of complex numbers                  */
/*----------------------------------------------------------------------*/

SmC (result,r, c)
complex *result;
double r;
complex *c;
{
   result->r = r*c->r;
   result->i = r*c->i;
}

/*----------------------------------------------------------------------*/
/* .. CmC -- complex multimplication                                    */
/*----------------------------------------------------------------------*/

CmC (result, c1, c2)
complex *result,*c1,*c2;
{
   result->r = c1->r*c2->r - c1->i*c2->i;
   result->i = c1->r*c2->i + c1->i*c2->r;
}

/*----------------------------------------------------------------------*/
/* .. DC -- complex division                                            */
/*----------------------------------------------------------------------*/

DC (result,n, d)
complex *result, *n, *d;
{
   double dkd;

   dkd = d->r*d->r + d->i*d->i;  
   result->r = (n->r*d->r + n->i*d->i)/dkd;
   result->i = (n->i*d->r - n->r*d->i)/dkd;
}

/*----------------------------------------------------------------------*/
/* .. PowC -- raises a complex number to a power                        */
/*----------------------------------------------------------------------*/

PowC (result,z, n)
complex *result,*z;
double n;
{
   double theta, r;

   r = sqrt(z->r*z->r + z->i*z->i);
   theta = acos(z->r/r);   
   if (z->i < 0) theta = theta + TT;

   theta = theta * n;
   r = pow (r, n);
   result->r = r*cos(theta);
   result->i = r*sin(theta);
}
 
/*----------------------------------------------------------------------*/
/* .. MagC -- returns the magnitude of the complex number               */
/*----------------------------------------------------------------------*/

double MagC (c)
complex *c;
{
   double mag;

   mag = sqrt (c->r*c->r + c->i*c->i); 

   return mag; 
}  

