/*----------------------------------------------------------------------*/
/* dstackq --double stack for tree traversal for computing limit sets of*/
/*          kleinian groups--the quasifuchsian case.                    */
/*----------------------------------------------------------------------*/
/*             morey@math.ubc.ca - Jim Morey - June 1, 1995             */
/*----------------------------------------------------------------------*/

#include <stdio.h>

/* #define  BUGS 1*/
#define MAX_DEPTH      1001
#define NUM_STATE      5
#define ZERO           0.0000000001
#define MAX_LIST       30

/*-types----------------------------------------------------------------*/

typedef struct {
    double r;
    double i;
} complex;

double jabs();

typedef complex mobius[2][2];

struct branch { 
  mobius mob;
  int st,depth;
};

struct holding { 
  int st,depth;
};

struct example { 
  mobius T[4];
  int num[4];
  complex fp[4][MAX_LIST];
};

/*-globals--------------------------------------------------------------*/

struct branch current[MAX_DEPTH*2];
struct holding list[MAX_DEPTH*NUM_STATE];
struct example K;

/*----------------------------------------------------------------------*/
/*----------------------------------------------------------------------*/

main(argc,argv)
int argc;
char *argv[];
{
  int i,j,k,st,cur=0,list_pos=0,depth,num_options,draw_branch;
  double cutoff,big,epsilon,epsilon2;
  complex z[MAX_LIST];
  char *app_options[] = { "-depth","-cutoff","-eps","-eps2","-start","-end"};

  num_options = sizeof(app_options)/sizeof(char *);
  read_in_example();
  depth = 100;
  cutoff = 50.0;
  epsilon = 0.05;
  epsilon2 = 0.05;

  z[0].r = 1.0;  z[0].i = 0.0; 
  z[1].r = 1.0;  z[1].i = 0.0; 
  z[2].r = 1.0;  z[2].i = 0.0; 
  z[3].r = 1.0;  z[3].i = 0.0; 
  z[4].r = 1.0;  z[4].i = 0.0; 
  current[0].st = 4;
  current[0].depth = 0;
  current[0].mob[0][0].r = 1.0;  current[0].mob[0][0].i = 0.0; 
  current[0].mob[0][1].r = 0.0;  current[0].mob[0][1].i = 0.0; 
  current[0].mob[1][0].r = 0.0;  current[0].mob[1][0].i = 0.0; 
  current[0].mob[1][1].r = 1.0;  current[0].mob[1][1].i = 0.0; 

  printf("1\n0\n");  /* .. tell dump -- lines + infinite number of entries ..*/

  /* .. read in the arguements .. */

  for (i=1;i<argc;i++) 
    for(j=0;j<num_options;j++) 
      if(strcmp(app_options[j],argv[i]) ==0) 
        switch (j){
          case 0:
            sscanf(argv[i+1],"%d",&depth);
            i++;
          break;
          case 1:
            sscanf(argv[i+1],"%lf",&cutoff);
            i++;
          break;
          case 2:
            sscanf(argv[i+1],"%lf",&epsilon);
            i++;
          break;
          case 3:
            sscanf(argv[i+1],"%lf",&epsilon2);
            i++;
          break;
          case 4:
            for(k=0;*(argv[i+1]+k) != 0; k++){
              list[list_pos].st = *(argv[i+1]+k) - '0';
              list[list_pos].depth = k+1;
              list_pos++;
              pop_push(&cur,&list_pos);
            }
            i++;
          break;
        }

  /* .. start traversing .. */

  forward(&list_pos,current[cur].st,current[cur].depth);
  while (list_pos != 0){
    pop_push(&cur,&list_pos);
    st = current[cur].st;
    /* .. not quite but similar to the Matrix norm .. */
    big = jabs(current[cur].mob[0][0].r) + jabs(current[cur].mob[0][0].i) +
          jabs(current[cur].mob[0][1].r) + jabs(current[cur].mob[0][1].i) +
          jabs(current[cur].mob[1][0].r) + jabs(current[cur].mob[1][0].i) +
          jabs(current[cur].mob[1][1].r) + jabs(current[cur].mob[1][1].i);
  
    if (big>cutoff){ 
      /* .. calculate fixed points and see if we should draw the branch .. */
      draw_branch = 1;
      zmob(&z[0],&K.fp[st][0],current[cur].mob);
     
      for (i=1;i<K.num[st];i++){
        zmob(&z[i],&K.fp[st][i],current[cur].mob);
        if (jabs(z[i].r-z[i-1].r) + jabs(z[i].i-z[i-1].i)> epsilon2)
          draw_branch = 0;
      }
      if (jabs(z[0].r-z[i-1].r)+jabs(z[0].i-z[i-1].i)> epsilon)
        draw_branch = 0;
    }
    else  draw_branch = 0; 

    if ((current[cur].depth <depth) && !draw_branch){
      forward(&list_pos,current[cur].st,current[cur].depth);
    } else if (draw_branch) {
      printf("%d %d\n",K.num[st], cur+1);
      for (i=0;i<K.num[st];i++)
        printf("%9.6f %9.6f\n",z[i].r,z[i].i);
    }
  }
}

/*----------------------------------------------------------------------*/

forward(list_pos,state,depth)
int *list_pos;
int state,depth;
{
  int i;
  static int arrows[NUM_STATE][NUM_STATE] = {{1,1,0,1,0},
                                             {1,1,1,0,0},
                                             {0,1,1,1,0},
                                             {1,0,1,1,0},
                                             {1,1,1,1,0}};
  
  for(i=0;i<NUM_STATE;i++){
  
    if (arrows[state][i] == 1) {
      list[*list_pos].st = i;
      list[*list_pos].depth = depth+1;
      (*list_pos)++;
    } 
  }
}

/*----------------------------------------------------------------------*/

pop_push(cur,list_pos)
int *cur,*list_pos;
{
  int j,k;
  complex t1;

  while (current[*cur].depth>=list[(*list_pos)-1].depth) (*cur)--;
  (*cur)++;
 
  current[*cur].st = list[(*list_pos)-1].st;
  current[*cur].depth = list[(*list_pos)-1].depth;
  for(j=0;j<2;j++)
    for(k=0;k<2;k++){
      current[*cur].mob[k][j].r = 
        current[(*cur)-1].mob[k][0].r*K.T[current[*cur].st][0][j].r 
        - current[(*cur)-1].mob[k][0].i*K.T[current[*cur].st][0][j].i 
        + current[(*cur)-1].mob[k][1].r*K.T[current[*cur].st][1][j].r 
        - current[(*cur)-1].mob[k][1].i*K.T[current[*cur].st][1][j].i;
      current[*cur].mob[k][j].i = 
        current[(*cur)-1].mob[k][0].r*K.T[current[*cur].st][0][j].i 
        + current[(*cur)-1].mob[k][0].i*K.T[current[*cur].st][0][j].r 
        + current[(*cur)-1].mob[k][1].r*K.T[current[*cur].st][1][j].i 
        + current[(*cur)-1].mob[k][1].i*K.T[current[*cur].st][1][j].r;
    }

  (*list_pos)--;
}

/*----------------------------------------------------------------------*/

zmob(nz,z,T)
complex *nz,*z;
mobius T;
{
  complex a,b;
  double bb;

  a.r = T[0][0].r * z->r - T[0][0].i * z->i + T[0][1].r;
  a.i = T[0][0].r * z->i + T[0][0].i * z->r + T[0][1].i;

  b.r = T[1][0].r * z->r - T[1][0].i * z->i + T[1][1].r;
  b.i = T[1][0].r * z->i + T[1][0].i * z->r + T[1][1].i;

  bb =  b.r * b.r +  b.i * b.i;
  nz->r = (a.r * b.r + a.i * b.i)/bb;
  nz->i = (a.i * b.r - a.r * b.i)/bb;
}
  
/*----------------------------------------------------------------------*/

double jabs(x)
double x;
{
  if (x>0.0) return x;
  else return -x;
}

/*----------------------------------------------------------------------*/

read_in_example()
{
  int i,j,k;

  /* .. read in T0 to T4 .. */

  for(i=0;i<4;i++)
    for(j=0;j<2;j++)
      for(k=0;k<2;k++){
        scanf("%lf",&K.T[i][j][k].r);
        scanf("%lf",&K.T[i][j][k].i);
      }

  /* .. read in the corresponding fixed points .. */

  for(i=0;i<4;i++){
    scanf("%d",&(K.num[i]));
    for(j=0;j<K.num[i];j++){
      scanf("%lf",&(K.fp[i][j].r));
      scanf("%lf",&(K.fp[i][j].i));
    }
  }
}
